﻿using AirplaneWithRadar.PaintObjects;
using AirplaneWithRadar.MovementStrategy;

namespace AirplaneWithRadar.Generics
{
    internal class AirplanesGenericCollection<T, U>
        where T : PaintAirplane
        where U : IMoveableObject
    {
        private readonly int pictWidth;
        private readonly int pictHeight;
        private readonly int placeSizeWidth = 200;
        private readonly int placeSizeHeight = 70;
        private readonly SetGeneric<T> collection;

        public void SortSet(IComparer<T?> comparer) => collection.SortSet(comparer);

        public AirplanesGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / placeSizeWidth;
            int height = picHeight / placeSizeHeight;
            pictWidth = picWidth;
            pictHeight = picHeight;
            collection = new SetGeneric<T>(width * height);

        }
        public static int? operator +(AirplanesGenericCollection<T, U> collect, T? obj)
        {
            if (obj == null)
            {
                return -1;
            }
            return collect?.collection.Insert(obj, new PaintAirplaneEqutables());
        }
        public static bool operator -(AirplanesGenericCollection<T, U> collect, int pos)
        {
            T? obj = collect.collection[pos];
            return collect.collection.Remove(pos);
        }

        public U? GetU(int pos)
        {
            return (U?)collection[pos]?.GetMoveableObject;
        }

        public Bitmap ShowAirplanes()
        {
            Bitmap bmp = new(pictWidth, pictHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }

        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < pictWidth / placeSizeWidth; i++)
            {
                for (int j = 0; j < pictHeight / placeSizeHeight + 1; ++j)
                {
                    g.DrawLine(pen, i * placeSizeWidth, j * placeSizeHeight, i * placeSizeWidth + placeSizeWidth / 2, j *  placeSizeHeight);
                }
                g.DrawLine(pen, i * placeSizeWidth, 0, i * placeSizeWidth, pictHeight / placeSizeHeight * placeSizeHeight);
            }
        }
        private void DrawObjects(Graphics g)
        {
            int width = pictWidth / placeSizeWidth;
            int height = pictHeight / placeSizeHeight;

            int j = 3;
            int k = 0;

            foreach (var airplane in collection.GetAirplanes()) {
                if (j < 0)
                {
                    j += 4;
                    k++;
                }
                if (airplane != null)
                {
                    airplane.SetPosition(placeSizeWidth * j, placeSizeHeight * k);
                    airplane.DrawTransport(g);
                }
                j--;
            }
        }
        public IEnumerable<T?> GetAirplanes => collection.GetAirplanes();
    }
}
