﻿using AirplaneWithRadar.PaintObjects;
using AirplaneWithRadar.MovementStrategy;
using System.Text;
using AirplaneWithRadar.Exceptions;

namespace AirplaneWithRadar.Generics
{
    internal class AirplanesGenericStorage
    {

        readonly Dictionary<AirplaneCollectionInfo, AirplanesGenericCollection<PaintAirplane, PaintObjectAirplane>> airplaneStorages;
        public List<AirplaneCollectionInfo> Keys => airplaneStorages.Keys.ToList();
        private readonly int pictWidth;
        private readonly int pictHeight;

        private static readonly char separatorForKeyValue = '|';
        private readonly char separatorRecords = ';';
        private static readonly char separatorForObject = ':';
        public AirplanesGenericStorage(int pictureWidth, int pictureHeight)
        {
            airplaneStorages = new Dictionary<AirplaneCollectionInfo, AirplanesGenericCollection <PaintAirplane, PaintObjectAirplane>>();
            pictWidth = pictureWidth;
            pictHeight = pictureHeight;
        }
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<AirplaneCollectionInfo, AirplanesGenericCollection<PaintAirplane, PaintObjectAirplane>> record in airplaneStorages)
            {
                StringBuilder records = new();
                foreach (PaintAirplane? elem in record.Value.GetAirplanes)
                {
                    records.Append($"{elem?.GetDataForSave(separatorForObject)}{separatorRecords}");
                }
                data.AppendLine($"{record.Key}{separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                throw new Exception("Невалиданая операция, нет данных для сохранения");
            }
            string strs = data.ToString();
            using (StreamWriter sr = new StreamWriter(filename)) {
                sr.WriteLine("AirplaneStorage");
                sr.WriteLine(strs);
            }
            return;
        }
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new Exception("Файл не найден");
            }
            using (StreamReader reader = new StreamReader(filename))
            {
                string checker = reader.ReadLine();
                if (checker == null)
                {
                    throw new Exception("Нет данных для загрузки");
                }
                if (!checker.StartsWith("AirplaneStorage"))
                {
                    throw new Exception("Неверный формат данных");
                }
                airplaneStorages.Clear();
                string strs;
                bool firstinit = true;
                while ((strs = reader.ReadLine()) != null)
                {
                    if (strs == null && firstinit)
                        return;
                    if (strs == null)
                        break;
                    if (strs == String.Empty)
                        break;
                    firstinit = false;
                    string name = strs.Split('|')[0];
                    AirplanesGenericCollection<PaintAirplane, PaintObjectAirplane> collection = new(pictWidth, pictHeight);
                    foreach (string data in strs.Split('|')[1].Split(';'))
                    {
                        PaintAirplane? airplane =
                        data?.CreatePaintAirplane(separatorForObject, pictWidth, pictHeight);
                        if (airplane != null)
                        {
                            try {
                               _ = collection + airplane;
                            }
                            catch (AirplaneNotFoundException e)
                            {
                                throw e;
                            }
                            catch (StorageOverflowException e)
                            {
                                throw e;
                            }
                        }
                    }
                    airplaneStorages.Add(new AirplaneCollectionInfo(name, String.Empty), collection);
                }
                return;
            }
        }

        public void AddSet(string name)
        {
            var newColl = new AirplanesGenericCollection<PaintAirplane, PaintObjectAirplane>(pictWidth, pictHeight);
            if (Keys.Contains(new AirplaneCollectionInfo(name, string.Empty))) 
            {
                MessageBox.Show("Набор с таким именем уже существует", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (airplaneStorages.ContainsKey(new AirplaneCollectionInfo(name, String.Empty))) { return; }
            airplaneStorages.Add(new AirplaneCollectionInfo(name, String.Empty), newColl);
        }
        public void DelSet(string name)
        {
            if (airplaneStorages.ContainsKey(new AirplaneCollectionInfo(name, String.Empty)))
                airplaneStorages.Remove(new AirplaneCollectionInfo(name, String.Empty));
        }
        public AirplanesGenericCollection<PaintAirplane, PaintObjectAirplane>? this[string ind]
        {
            get
            {
                if (!airplaneStorages.ContainsKey(new AirplaneCollectionInfo(ind, String.Empty)))
                    return null;
                return airplaneStorages[new AirplaneCollectionInfo(ind, String.Empty)];
            }
        }
    }

}
