﻿using AirplaneWithRadar.PaintObjects;
using AirplaneWithRadar.Entities;

namespace AirplaneWithRadar
{
    public partial class FormAirplaneConfig : Form
    {
        PaintAirplane? airplane = null;

        private event Action<PaintAirplane> EventAddAirplane;
        public FormAirplaneConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            buttonCancel.Click += (sender, e) => Close();
        }

        public void DrawAirplane()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            airplane?.SetPosition(5, 5);
            airplane?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        public void AddEvent(Action<PaintAirplane> ev)
        {
            if (EventAddAirplane == null)
            {
                EventAddAirplane = new Action<PaintAirplane>(ev);
            }
            else
            {
                EventAddAirplane += ev;
            }
        }

        private void labelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    airplane = new PaintAirplane((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    airplane = new PaintAirplaneWithRadar((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxRadar.Checked,
                   checkBoxAdditFuelPod.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawAirplane();
        }

        private void labelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (airplane != null && airplane.AirplaneEntity is AirplaneWithRadarEntity entityAirplane)
            {
                labelAdditColor.AllowDrop = true;
            }
            else
            {
                labelAdditColor.AllowDrop = false;
            }
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void labelMainColor_DragDrop(object sender, DragEventArgs e)
        {
            if (airplane != null)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    airplane.AirplaneEntity.BodyColor = (Color)e.Data.GetData(typeof(Color));
                }
                DrawAirplane();
            }
        }

        private void labelAdditColor_DragDrop(object sender, DragEventArgs e)
        {
            if (airplane != null && airplane.AirplaneEntity is AirplaneWithRadarEntity airplaneWithRadarEntity)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    airplaneWithRadarEntity.AdditColor = (Color)e.Data.GetData(typeof(Color));
                }
                DrawAirplane();
            }
        }

        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            EventAddAirplane?.Invoke(airplane);
            Close();
        }
    }
}