using System;
using System.Drawing;
using AirplaneWithRadar.MovementStrategy;
using AirplaneWithRadar.PaintObjects;

namespace AirplaneWithRadar
{
    public partial class PlaneVisual : System.Windows.Forms.Form
    {
        private PaintAirplane? PaintPlanes;
        private AbstractStrategy? abstractStrategy;
        public PaintAirplane? SelectedAirplane { get; private set; }
        private void Draw()
        {
            if (PaintPlanes == null)
            {
                return;
            }
            Bitmap bitmap = new Bitmap(PlanesPictureBox.Width, PlanesPictureBox.Height);
            Graphics gr = Graphics.FromImage(bitmap);
            PaintPlanes.DrawTransport(gr);
            PlanesPictureBox.Image = bitmap;
        }
        public PlaneVisual()
        {
            InitializeComponent();
            abstractStrategy = null;
            SelectedAirplane = null;
        }

        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (PaintPlanes == null) { return; }
            string name = ((Button)sender)?.Name ?? string.Empty;

            switch (name)
            {
                case "UpButton":
                    PaintPlanes.Move(Movement.Up);
                    break;
                case "DownButton":
                    PaintPlanes.Move(Movement.Down);
                    break;
                case "LeftButton":
                    PaintPlanes.Move(Movement.Left);
                    break;
                case "RightButton":
                    PaintPlanes.Move(Movement.Right);
                    break;

            }
            Draw();
        }

        private void CreateButton_Click(object sender, EventArgs e)
        {

            Random random = new Random();

            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }

            Color dopColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialog.Color;
            }

            PaintPlanes = new PaintAirplaneWithRadar(random.Next(100, 300), random.Next(1000, 3000), color, dopColor, true,
                Convert.ToBoolean(random.Next(0, 2)), PlanesPictureBox.Width, PlanesPictureBox.Height);
            PaintPlanes.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void CreateAirplaneButton_Click(object sender, EventArgs e)
        {
            Random random = new Random();

            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            PaintPlanes = new PaintAirplane(random.Next(100, 300), random.Next(1000, 3000), color,
                PlanesPictureBox.Width, PlanesPictureBox.Height);
            PaintPlanes.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void StepButton_Click(object sender, EventArgs e)
        {
            if (PaintPlanes == null)
            {
                return;
            }
            if (StrategyComboBox.Enabled)
            {
                abstractStrategy = StrategyComboBox.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null
                };
                if (abstractStrategy == null)
                {
                    return;
                }
                abstractStrategy.SetData(new PaintObjectAirplane(PaintPlanes), PlanesPictureBox.Width, PlanesPictureBox.Height);
                StrategyComboBox.Enabled = false;
            }
            if (abstractStrategy == null)
            {
                return;
            }
            abstractStrategy.MakeStep();
            Draw();
            if (abstractStrategy.GetStatus() == Status.Finish)
            {
                StrategyComboBox.Enabled = true;
                abstractStrategy = null;
            }
        }

        private void buttonSelectAirplane_Click(object sender, EventArgs e)
        {
            SelectedAirplane = PaintPlanes;
            DialogResult = DialogResult.OK;

        }
    }
}